;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                            ;;;
;;;  Subtraction & Typing experiment                           ;;;
;;;                                                            ;;;
;;;  Trudy Buwalda (t.a.buwalda@student.rug.nl) &              ;;;
;;;  Jelmer Borst (jpborst@ai.rug.nl)                          ;;;
;;;  Artificial Intelligence                                   ;;;
;;;  University of Groningen                                   ;;;
;;;  The Netherlands                                           ;;;
;;;                                                            ;;;
;;;  20101005                                                  ;;;
;;;                                                            ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                                                                                                     

;macro for safely evaluating stuff - if bus error, just continue till there's no error anymore
(defmacro safe-exp ((type) &body body)
  `(let ((rv nil))
     (handler-case
         (progn
           (setf rv (progn ,@body)))
        (error ()
          (progn
            (write-error ,type))))
    rv))


(defparameter *error-name-exp* "errors-exp")


;print errors to error file. set this filename by calling (set-error)
(defun set-error-exp (name)
  (setf *error-name-exp* name))

(defun write-error (type)
  (with-open-file (out (ensure-directories-exist 
                         (merge-pathnames 
                          (make-pathname :name *error-name-exp* :directory '(:relative "data") :type "txt") 
                          (current-pathname)))
                        :direction :output :if-does-not-exist :create :if-exists :append)
    (format out "~%error in ~a" type)))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Parameter definitions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;Experiment flow parameters
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defparameter *eye* nil) ;eyetracking?
(defparameter *eye-test* nil) ;offline eyetracker!
(defparameter *model* t)


(let ((file (merge-pathnames 
             (make-pathname :name "eyelink" :type "lisp")  
             (current-pathname))))
  (load file))
      

(defparameter *signal* t) ; are we using the signal variant?

(defparameter *block* 0) ;current block in the experiment
(defparameter *trial* 0)
(defparameter *step* 0)

(defparameter *waiting* nil)
(defparameter *sub_step* 0)
(defparameter *typ_step* 0)

(defparameter *exp* nil) ;the experiment object


;;moving the mouse cursor
(fli:define-c-typedef cg-float
 #-lispworks-64bit :float
 #+lispworks-64bit :double)

(fli:define-c-struct cgpoint (x cg-float) (y cg-float))
(fli:define-c-typedef point (:struct cgpoint))

(fli:define-foreign-function (set-cursor-position "CGWarpMouseCursorPosition") ((newCursorPosition cgpoint)))

(defun set-mouse-pos (x y)
  (fli:with-dynamic-foreign-objects ()
    (let ((location (fli:allocate-dynamic-foreign-object :type 'point)))
      (setf (fli:foreign-slot-value location 'x) (/ x 1.0))
      (setf (fli:foreign-slot-value location 'y) (/ y 1.0))
      (set-cursor-position (fli:dereference location :copy-foreign-object nil)))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; File Handling & participant setup - .../Lisp snippets/file handlers.lisp
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defparameter *pp* 99) ;participant#
(defparameter *exp-part* "test") ;name of the experiment part for use in filename

(defparameter *output-pathname* ;pathname to outputfile, default "pp-test"
  (merge-pathnames 
   (make-pathname :name "pp-test" :directory '(:relative "data") :type "txt")  
   (current-pathname))) ;use current path to start with, put it in relative directory 'data'

(defparameter *output-stream* nil) ;stream to write to
(defparameter *output-buffer* nil) ;buffer for output
(defparameter *output-buffer-size* 0) ;length of output buffer list
(defparameter *overwrite-p* nil) ;overwrite file if exist yes/no

;;ask for pp#, if exists, ask for overwrite confirmation or for new #
(defun setup-pp ()
   (setf *pp* (loop
              (let ((pp (capi:prompt-for-integer "Please enter a participant number:")))
                (cond 
                  ((probe-file ;if file exist
                    (setf *output-pathname* 
                          (make-pathname :name (format nil "~a-pp~a" *exp-part* pp) :defaults *output-pathname*)))
                   (when (capi:confirm-yes-or-no "Participant # already in use, overwrite and proceed anyway?") ;ask what to do
                     (setf *overwrite-p* t)
                     (return pp)))
                  (t
                   (return pp)))))))

;;;open the file to write and start with pp#, date and "(setf *data* '(", for easy reading later
(defun initialize-outfile ()
  (setf *output-pathname* (make-pathname :name (format nil "~a-pp~a" *exp-part* *pp*) :defaults *output-pathname*))
  (do ((i 1 (incf i)))
      ((setf *output-stream* ;actually don't overwrite when file already exists, but append -#.
             (open 
              (ensure-directories-exist *output-pathname*) 
              :direction :output 
              :if-does-not-exist :create 
              :if-exists (if *overwrite-p* ':supersede nil))))
    (setf *output-pathname* (make-pathname :name (format nil "~a-pp~a-~a" *exp-part* *pp* i) :defaults *output-pathname*)))
  (format *output-stream* "; Participant number: ~a~%" *pp*)
  (let ((time-list (multiple-value-list (get-decoded-time))))
    (format *output-stream* "; ~2,'0d:~2,'0d, ~d/~d/~2,'0d (mm/dd/yy)~%~%" 
            (third time-list) 
            (second time-list)
            (fifth time-list)
            (fourth time-list)
            (sixth time-list)))
  (format *output-stream* "(setf *data* '("))


;;write to buffer, if size exceeds 10: flush buffer to file
(defun print-outfile (string &rest args)
  (let ((string (apply #'format (append (list nil (substitute #\Tab #\Space string)) args))))
    (push string *output-buffer*))
  (incf *output-buffer-size*)
  (if (> *output-buffer-size* 10) (flush-buffer)))


;;flush buffer to file
(defun flush-buffer ()
  (dolist (string (reverse *output-buffer*))
    (format *output-stream* "~&")
    (format *output-stream* string))
  (setf *output-buffer* nil)
  (setf *output-buffer-size* 0)
  (gc-if-needed)) ;garbage cleaning if needed


;;flush buffer & close file and end the file with "))"
(defun finalize-outfile ()
  (flush-buffer)
  (format *output-stream* "))")
  ;(format *output-stream* "~%;points: ~a" (points *exp*))
  (close *output-stream*))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Supporting functions - switch on what's necessary
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;Zorgt ervoor dat de lispworks toolbar niet zichtbaar is
(defun kill-toolbar ()
 (let ((interfaces (capi:collect-interfaces 'LISPWORKS-TOOLS::LISPWORKS-TOOLBAR-WINDOW))) ;destroy the lispworks toolbar!
    (if interfaces (mapcar #'(lambda (interface) 
                               (capi:apply-in-pane-process interface 'capi:destroy interface))
                           interfaces))))

(defun randomize (lst)		;list randomizer
  (let ((lst (copy-list lst)))
    (dotimes (i (length lst))
      (let ((x (nth i lst))
            (ni (+ i (random (- (length lst) i)))))
        (setf (nth i lst) (nth ni lst))
        (setf (nth ni lst) x)))
    lst))


;;returns time to use, real-time in case of human pp
(defmacro my-get-time ()  `(if *model* (* (mp-time) 1000) (get-internal-real-time)))

(defun my-tracker-time () (if *eye* (safe-exp ("get tracker time") (eyelink:eyelink-time)) -1))

(defmacro my/ (x y) `(if (zerop ,y) 0 (/ ,x ,y)))	;my div, 0 if div by 0
(defmacro sqr (x) `(let ((v ,x)) (* v v)))		;square

(defun scramble (sequence) ;scrambles any sequence, including strings: "jelmer" -> "eljrme"
  (setf sequence (copy-seq sequence))
  (let ((length (length sequence)))
    (dotimes (i (1- length) sequence)
      (rotatef
       (elt sequence i)
       (elt sequence (random (+ i (- length  i))))))))

;;prints any list, one item per line
(defun print-list (lst)
  (dolist (i lst) (print i)))

;;calculates number of different characters for two strings, used for calculating mistakes
;;
;;Only used for calculating mistakes, not points
(defun diff (string1 string2)
  (let ((diffscore 0))
    (dotimes (i (length string1))
      (if (not (equalp (char string1 i) (char string2 i))) (incf diffscore)))
    diffscore))

;;calculate edit-distance between two words
(defun edit-distance (word1 word2)
  (let ((d (make-array (list 32 32) :element-type 'fixnum)))
    (declare (type simple-string word1 word2)
             (optimize (speed 3)))
    (let ((length1 (length word1))
          (length2 (length word2)))
      (loop for i from 0 to length1
            do (setf (aref d i 0) i))
      (loop for j from 0 to length2
            do (setf (aref d 0 j) j))
      (loop for i from 1 to length1
            do (loop for j from 1 to length2
                     do (setf (aref d i j)
                              (min (1+ (aref d (1- i) j))
                                   (1+ (aref d i (1- j)))
                                   (+ (aref d (1- i) (1- j))
                                      (if (char= (aref word1 (1- i))
                                                 (aref word2 (1- j)))
                                          0
                                        1))))))
      (aref d length1 length2))))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; ACT-R helpers
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;starts/stops act-r environment
(defun se ()
  (if *environment-sockets* (stop-environment) (start-environment)))

;;runs/stops act-r environment
(defun re ()
  (if *environment-sockets* (stop-environment) (run-environment)))

;; show act-r trace
(defmethod show-trace ()
  (format t "Press enter to return to lispworks.")
  (read-line *terminal-io*))

;;schedule dummy to indicate visual change
(defun schedule-dummy (&optional (rel 0))
  (schedule-event-relative rel (lambda (module) (progn module)) :maintenance nil :destination 'procedural :details "Dummy to signal visual change"))


;;generate easy stimuli for subtraction, no carrying/borrowing.
(defun get-easy-stims (nr)
  (let ((lst nil))
    (dotimes (i nr)
      (push 
       (let ((A "0")
             (B "0"))
         (do () ((and ;keep trying to make such numbers until all terms are 10 digits long
                  (equalp (length (write-to-string (parse-integer A))) 10)
                  (equalp (length (write-to-string (parse-integer B))) 10)
                  (equalp (length (write-to-string (- (parse-integer A) (parse-integer B)))) 10)))
           (setf A "")
           (setf B "")
           (dotimes (i 10)
             (let ((termA (random 10))
                   (termB (random 10)))
               (do () ((> termA termB)) ;choose random numbers, make sure B < A
                 (setf termA (random 10))
                 (setf termB (random 10)))
               (setf A (format nil "~a~a" termA A))
               (setf B (format nil "~a~a" termB B)))))
         (list (parse-integer A) (parse-integer B)))
       lst))
    lst))

;;calculate how many times a carry is necessary in a subtraction
(defun calculate-carries (termA termB)
  (let ((A (reverse (format nil "~a" termA)))
        (B (reverse (format nil "~a" termB)))
        (prev1 "")
        (prev2 "")
        (carries 0))
    (loop 
     for dig1 across A
     for dig2 across B
     do
     (setf dig1 (parse-integer (format nil "~a" dig1)))
     (setf dig2 (parse-integer (format nil "~a" dig2)))
     (if (< dig1 dig2) (incf carries)) ;if dig1 < dig2, add carry
     (if (equalp dig1 dig2) (+ carries 100)) ;;we don't want equal digits to be subtracted
     (when (and
            (equalp dig1 dig2) ;if both digs are the equal
            (> (length prev1) 0) ;there already a previous digit
            (or (< (- (parse-integer prev1) (parse-integer prev2)) 0) (equalp (subseq prev1 0 1) (subseq prev2 0 1)))) ;and the previous subtraction needs to carry
       (incf carries 15)) ;add 15 carries because we don't want this.     
     (if (and (> (length prev1) 0) ;in case dig1 == 0 and carry is necessary, increase carries by 10, as we don't want that
              (equalp dig1 0) 
              (< (- (parse-integer prev1) (parse-integer prev2)) 0))
         (incf carries 10))

     (setf prev1 (format nil "~a~a" dig1 prev1))
     (setf prev2 (format nil "~a~a" dig2 prev2)))
    carries))


;;generate hard stimuli for subtraction, ie 6 carries in a 10 digit subtraction
(defun get-hard-stims (nr)
  (let ((lst nil))
    (dotimes (i nr)
      (push 
       (let ((A "0")
             (B "0"))
         (do () ((and
                  (equalp (length (write-to-string (parse-integer A))) 10)
                  (equalp (length (write-to-string (parse-integer B))) 10)
                  (equalp (length (write-to-string (- (parse-integer A) (parse-integer B)))) 10)
                  (> (- (parse-integer A) (parse-integer B)) 0)
                  (equalp (calculate-carries (parse-integer A) (parse-integer B)) 6)))
           (setf A "")
           (setf B "")
           (dotimes (i 10)
             (let ((termA (random 10))
                   (termB (random 10)))
               (do () ((or (and (> (random 3) 1) (< termA termB)) (> termA termB)))
                 (setf termA (random 10))
                 (setf termB (random 10)))
               (setf A (format nil "~a~a" termA A))
               (setf B (format nil "~a~a" termB B)))))
         (equalp (calculate-carries A B) 6)
         (list (parse-integer A) (parse-integer B)))
       lst))
    lst))


        


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Stimuli
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;stimuli for hard typing 
(defparameter *typing-stimuli-hard*
  (randomize '("zorgvuldig"
               "gezelschap"
               "terugkomen"
               "uitstekend"
               "regelmatig"
               "ruimtelijk"
               "bewustzijn"
               "bevestigen"
               "instelling"
               "literatuur"
               "overtuigen"
               "gewoonlijk"
               "aanleiding"
               "historisch"
               "beschermen"
               "gevaarlijk"
               "toepassing"
               "voorwaarde"
               "fluisteren"
               "ziekenhuis"
               "verklaring"
               "onmogelijk"
               "beslissing"
               "activiteit"
               "schreeuwen"
               "informatie"
               "behandelen"
               "verhouding"
               "wetenschap"
               "vervolgens"
               "economisch"
               "betrekking"
               "glimlachen"
               "beschouwen"
               "nauwelijks"
               "plotseling"
               "antwoorden"
               "veranderen"
               "verdwijnen"
               "belangrijk"
               "natuurlijk"
              )))

;;stimuli for easy typing, scrambled hard stimuli without repeating same character
(defparameter *typing-stimuli-easy*
  (randomize '("gluozidgvr"
               "tiusbejzwn" 
               "winjervedn" 
               "suteilrfen" 
               "emgerkuotn" 
               "iocnocshem" 
               "sbesinsgil" 
               "iesaonpsgt" 
               "ebnlehnead" 
               "klsjweniau" 
               "uarluijtnk" 
               "iktemlruji" 
               "ilgrbnjaek" 
               "nrougdivhe" 
               "inksiezhue" 
               "zceahglsep" 
               "oegnvlvrse" 
               "cserenhmeb" 
               "iaticivtet" 
               "aigegtrmel" 
               "adilaenign" 
               "wenshatcpe" 
               "esbeinvetg" 
               "nioitfemar" 
               "avrarodweo" 
               "mleialghcn" 
               "aedtornown" 
               "tsoirihshc" 
               "tuiretlura" 
               "ptlsnloieg" 
               "engiolwjok" 
               "rktekinebg" 
               "esohcbuwne" 
               "dtenisektu" 
               "uercweshen" 
               "iruveontge" 
               "nljmoigkeo" 
               "instlielng" 
               "elkrivarng" 
               "naerdvenre"
               "valragijek" )))

;;easy subtraction stimuli

(defparameter *subtraction-stimuli-easy*
  (get-easy-stims 41))

;;hard subtraction stimuli

(defparameter *subtraction-stimuli-hard*
  (get-hard-stims 41))

     
;;De huidige levels
(defparameter *current-levels* nil) 
                               




;;;
;;;  Simulation Items // ACT-R
;;; 

;;upper item for all simulation items
(defclass st-window-item ()
  ((x :accessor x :initform 0 :initarg :x)
   (y :accessor y :initform 0 :initarg :y)
   (color :accessor color :initform 0 :initarg :color)
   (height :accessor height :initform 0 :initarg :height)
   (width :accessor width :initform 0 :initarg :width)
   (kind :accessor kind :initarg :kind)))

;text-feature, chunks will have the following slots, to be filled later
(defclass text-feature (st-window-item)
  ((text :accessor text :initform "" :initarg :text))
  (:default-initargs
      :kind 'text
      :height 10
      :width 100
      :color 'black))

(defclass circle-feature (st-window-item)
  ((circle :accessor circle :initform ""))
  (:default-initargs
      :kind 'circle
      :height 60
      :width 60
      :color 'green))

;etc.
(defclass button-feature (st-window-item)
  ((label :accessor label :initform "" :initarg :label)
   (enabled :accessor enabled :initform t :initarg :enabled))
  (:default-initargs
      :kind 'button
      :height 20
      :width 48
      :color 'black))


(defparameter *typing-on* nil)
(defparameter *subtraction-on* nil)

;;;lists of features to be added to the visicon when necessary
(defparameter *alphabet-buttons* nil)
(defparameter *numpad-buttons* nil)

(defparameter *typing-input-feature* nil)
(defparameter *subtraction-A-feature* nil)
(defparameter *subtraction-B-feature* nil)
(defparameter *subtraction-C-feature* nil)

(defparameter *signal-feature* nil)


(defparameter *sub-circle-feature* nil)
(defparameter *typ-circle-feature* nil)

;;simulation for ACT-R
(defun create-features ()
  (setf *alphabet-buttons*
        (mapcar 
         #'(lambda (label x y) (make-instance 'button-feature :x (+ 630 x) :y (+ 555 y) :label label)) 
           (list "A" "B" "C" "D" "E" "F" "G" ;labels
                 "H" "I" "J" "K" "L" "M" "N" 
                  "O" "P" "Q" "R" "S" "T" "U" 
                   "V" "W" "X" "Y" "Z")
           (list 10 60 110 160 210 260 310  ;x-coords
                 10 60 110 160 210 260 310
                 10 60 110 160 210 260 310 
                 60 110 160 210 260)
           (list 5 5 5 5 5 5 5 ;y-coords
                 33 33 33 33 33 33 33
                 61 61 61 61 61 61 61
                 89 89 89 89 89)))
  (setf *numpad-buttons*
        (mapcar 
         #'(lambda (label x y) (make-instance 'button-feature :x (+ 135 x) :y (+ 555 y) :label label)) 
         (list "1" "2" "3" ;labels
               "4" "5" "6"  
               "7" "8" "9"  
               "0")
         (list 10 60 110  ;x-coords
               10 60 110
               10 60 110
               60 )
         (list 5 5 5 ;y-coords
               33 33 33
               61 61 61 
               89 89 89 )))

  (setf *typing-input-feature*
        (make-instance 'text-feature :x 700 :y 190 :text "" :width 100))

  (setf *typ-circle-feature*
        (make-instance 'circle-feature :x 651 :y 334 ))
  (setf *sub-circle-feature*
        (make-instance 'circle-feature :x 367 :y 334 ))


  (setf *subtraction-A-feature*
        (make-instance 'text-feature :x 205 :y 135 :text "0"))
  (setf *subtraction-B-feature*
        (make-instance 'text-feature :x 205 :y 195 :text "0"))


  (setf *signal-feature* (make-instance 'text-feature :x 235 :y 100 :text "|"))

 ; (setf *subtraction-C-feature* (make-instance 'text-feature :x 460 :y 370 :text "0"))
)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Interface - .../Lisp snippets/interface.lisp
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;
;;;  Simulation Window
;;;

(defparameter *expwin* nil) ;window object containing everything

(defclass alphabet-panel (capi:push-button-panel) ;isa push button panel
  ()
  (:default-initargs
   :items (mapcar #'(lambda (text loc-x loc-y) ;create real buttons for exact positioning
                      (make-instance 'capi:push-button
                                     :text text 
                                     :visible-min-width 48                                   
                                   ;  :visible-min-height 10
                                     :max-width t
                                     :max-height t
                                     :data text
                                     :accepts-focus-p nil
                                     :x loc-x
                                     :y loc-y))
                  (list "A" "B" "C" "D" "E" "F" "G" ;labels
                        "H" "I" "J" "K" "L" "M" "N" 
                        "O" "P" "Q" "R" "S" "T" "U" 
                        "V" "W" "X" "Y" "Z")
                 (list 10 60 110 160 210 260 310  ;x-coords
                       10 60 110 160 210 260 310
                       10 60 110 160 210 260 310 
                       60 110 160 210 260)
                  (list 5 5 5 5 5 5 5 ;y-coords
                       33 33 33 33 33 33 33
                       61 61 61 61 61 61 61
                       89 89 89 89 89))
   :layout-class 'capi:pinboard-layout ;layout within push button panel
   :selection-callback 'process-input-button ;call input-button function when pressed
   :print-function 'string-capitalize
   :layout-args '(:background :white :visible-min-width 355 :visible-min-height 120))) ;args for panel

(defclass numpad-panel (capi:push-button-panel) ;isa push button panel
  ()
  (:default-initargs
   :items (mapcar #'(lambda (text loc-x loc-y) ;create real buttons for exact positioning
                      (make-instance 'capi:push-button
                                     :text text 
                                     :visible-min-width 48                                   
                                   ;  :visible-min-height 10
                                     :max-width t
                                     :max-height t
                                     :data text
                                     :accepts-focus-p nil
                                     :x loc-x
                                     :y loc-y))
                  (list "1" "2" "3" ;labels
                        "4" "5" "6"  
                        "7" "8" "9"  
                        "0")
                 (list 10 60 110  ;x-coords
                       10 60 110
                       10 60 110
                       60 )
                  (list 5 5 5 ;y-coords
                       33 33 33
                       61 61 61 
                       89 89 89 ))
   :layout-class 'capi:pinboard-layout ;layout within push button panel
   :selection-callback 'process-keypresses ;call input-button function when pressed
   :print-function 'string-capitalize
   :layout-args '(:background :white :visible-min-width 155 :visible-min-height 120))) ;args for panel


(defparameter *sub-circle-color* :white)
(defparameter *typ-circle-color* :white)
(defparameter *circles-on* nil)

(defun draw-subcircle (pane self x y width height)
  (declare (ignore self)
           (ignore width)
           (ignore height))
  (gp:draw-circle pane (+ x 30) (+ y 30) 29
                  :filled t 
                  :foreground *sub-circle-color*)
  (if *circles-on* (gp:draw-circle pane (+ x 30) (+ y 30) 29)))


(defun draw-typcircle (pane self x y width height)
  (declare (ignore self)
           (ignore width)
           (ignore height))
  (gp:draw-circle pane (+ x 30) (+ y 30) 29 
                  :filled t 
                  :foreground *typ-circle-color*)
  (if *circles-on* (gp:draw-circle pane (+ x 30) (+ y 30) 29)))

;drift target
(defun draw-driftcircle (pane self x y width height)
  (declare (ignore self)
           (ignore x)
           (ignore y)
           (ignore width)
           (ignore height))
  (gp:draw-circle pane 507 329 6 
                  :filled nil
                  :thickness 8
                  :foreground :black))

;; define the complete interface:
;;
;;
(capi:define-interface st-window ()
  ((subviews :accessor subs :initform nil) ;will hold object for ACT-R
   (mouse-pos :accessor mouse-pos :initform (vector 0 0))) ;mouse-pos used for act-r
  (:panes
   (msg capi:display-pane
        :accessor msg
        :text "                  Je kunt nu even je hoofd bewegen.
Druk op de spatiebalk wanneer je klaar bent om verder te gaan."
        :font (gp:make-font-description 
               :family "times"  
               :size 22
              ;; :pitch :fixed
               :weight :medium                         
               :slant :roman)
        :background :white
        :x 250
        :y 350)
   ;;de subtraction-circle
   (sub-circle capi:drawn-pinboard-object
                   :accessor sub-circle
                   :x 337
                   :y 304
                   :width 60
                   :height 65
                   :display-callback 'draw-subcircle)
   ;;de typing-circle 
   (typ-circle capi:drawn-pinboard-object
               :accessor typ-circle
               :x 621
               :y 304
               :width 60
               :height 65
               :display-callback 'draw-typcircle) 
   (drift-target capi:drawn-pinboard-object
               :accessor drift-target
               :x 0
               :y 0
               :width 6
               :height 6
               :display-callback 'draw-driftcircle)
   (empty-pane-typing capi:output-pane ;empty pane for when typing is hidden
                   :visible-min-width 170 
                   :visible-min-height 148
                   :background :white 
                   :accessor empty-pane-typing)
   (empty-pane-keyboard capi:output-pane ;empty pane for when subtraction is hidden
                        :visible-min-width 358 
                        :visible-min-height 123 
                        :background :white
                        :accessor empty-pane-keyboard)
   (empty-pane-numpad capi:output-pane ;empty pane for when subtraction is hidden
                        :visible-min-width 158
                        :visible-min-height 123 
                        :background :white
                        :accessor empty-pane-numpad)
   (input-screen-typing capi:text-input-pane ;typing text box - where the stimuli appear
                        :accessor input-screen-typing
                        :text ""
                        :enabled :read-only
                        :visible-min-width 210
                        :visible-min-height 25
                        :x 0
                        :y 50
                        :background :white
                        :font (gp:make-font-description 
                             ;  :family "times" 
                               :size 22
                              ; :pitch :fixed
                              :enabled :read-only
                               :weight :medium                         
                               :slant :roman)
                        :visible-border :outline)
   (alphabet alphabet-panel ;see class def above
             :accessor alphabet
             :background :white
             :x 5
             :y 5)
   (numpad numpad-panel ;see class def above
             :accessor numpad
             :background :white
             :x 5
             :y 5)
   (alphabet-border capi:rectangle ;border around typing-panel
               :accessor alphabet-border
               :x 620
               :y 540
               :width 380
               :height 140
               :graphics-args '(:foreground :black))
   (numpad-border capi:rectangle ;border around typing-panel
               :accessor numpad-border
               :x 125
               :y 540
               :width 180
               :height 140
               :graphics-args '(:foreground :black))
   (subtraction-line capi:line-pinboard-object ;line for subtraction
                     :accessor subtraction-line
                     :start-x 0
                     :start-y 155
                     :end-x 40
                     :end-y 155
                     :graphics-args '(:thickness 2) )
   (subtraction-minus capi:line-pinboard-object ;minus for subtraction
                     :accessor subtraction-minus
                     :start-x 51
                     :start-y 155
                     :end-x 64
                     :end-y 155
                     :graphics-args '(:thickness 2) )
   (empty-pane-subtraction capi:output-pane ;empty pane for when subtraction is hidden
                   :visible-min-width 150 
                   :visible-min-height 140 
                    :background :white
                   :accessor empty-pane-subtraction)
   (input-screen-subtraction capi:display-pane ;subtraction text box - where the response appears
                        :accessor input-screen-subtraction
                        :text "0"
                        :enabled :read-only
                        :visible-min-width 25
                        :visible-min-height 25
                        :background :white
                        :font (gp:make-font-description 
                        ;       :family "times" ;"andale mono" 
                               :size 22
                               :PITCH :FIXED ;;!! to make sure spacing is the always the same width, also see termA & B
                               :weight :medium                         
                               :slant :roman)
                        :x 10
                        :y 195
                        :visible-border :outline)
 
   (subtraction-signal capi:display-pane ;signal box - where the appears when there's a carrie
                        :accessor subtraction-signal
                        :text "1"
                        :background :white
                        :font (gp:make-font-description 
                        ;       :family "times" ;"andale mono" 
                               :size 14
                               :PITCH :FIXED ;;!! to make sure spacing is the always the same width, also see termA & B
                               :weight :medium                         
                               :slant :roman)
                        :x 0
                        :y 0)
   (subtraction-empty-signal capi:output-pane
                             :visible-min-width 10
                             :visible-min-height 10
                             :background :white
                             :accessor subtraction-empty-signal)

   (subtraction-input-border capi:rectangle     
                             :accessor subtraction-input-border
                             :x 5
                             :y 190
                             :width 35
                             :height 35
                             :graphics-args '(:foreground :black))
   (subtraction-termA capi:display-pane ;first term of subtraction
                      :accessor subtraction-termA
                      :text "0"
                      :font (gp:make-font-description 
                       ;      :family "times" 
                             :size 22
                             :PITCH :FIXED
                             :weight :medium                         
                             :slant :roman)
                       :background :white
                      :x 10
                      :y 35)
   (subtraction-termB capi:display-pane ;second term of subtraction
                      :accessor subtraction-termB
                      :text "0"
                      :font (gp:make-font-description 
                          ;   :family "times" 
                             :size 22
                             :pitch :fixed
                             :weight :medium                         
                             :slant :roman)
                       :background :white
                      :x 10
                      :y 95)
   (subtraction-correct capi:display-pane ;text panel for when subtraction is done correctly
                     :accessor subtraction-correct
                     :text "Correct!"
                     :font (gp:make-font-description 
                            :family "times" 
                            :size 30
                            :weight :medium                         
                            :slant :roman)
                     :foreground :darkgreen
                      :background :white
                     :x 0
                     :y 95)
   (subtraction-wrong capi:display-pane ;text panel for when subtraction is done correctly
                     :accessor subtraction-wrong
                     :text "Fout:("
                     :font (gp:make-font-description 
                            :family "times" 
                            :size 30
                            :weight :medium                         
                            :slant :roman)
                     :foreground :red
                     :background :white
                     :x 0
                     :y 95)
   (typing-correct capi:display-pane ;text panel for when typing is done correct
                :accessor typing-correct
                :text "Correct!"
                :font (gp:make-font-description 
                       :family "times" 
                            :size 30
                            :weight :medium                         
                            :slant :roman)
                :foreground :darkgreen
                 :background :white
                :x 30
                :y 55)
   (typing-wrong capi:display-pane ;text panel for when typing is done wrong
                :accessor typing-wrong
                :text "Fout:("
                :font (gp:make-font-description 
                       :family "times" 
                            :size 30
                            :weight :medium                         
                            :slant :roman)
                :foreground :red
                 :background :white
                :x 30
                :y 55)
    (fixation capi:display-pane
             :accessor fixation
             :text "+"
             :font (gp:make-font-description 
                    :family "times" 
                    :size 22
                    :pitch :fixed
                    :weight :medium                         
                    :slant :roman)
              :background :white
             :x 497
             :y 319
            
             :foreground :black )
 )


(:layouts

   ;;;meta-layout
   (meta-layout capi:switchable-layout
                '(fixation-layout main-layout msg-layout drift-layout)
                :accessor meta-layout
                
                )
   ;;;main-layout
   (main-layout capi:pinboard-layout  ;complete window layout
                '(top-layout subtraction-layout typing-layout keyboard-layout numpad-layout alphabet-border numpad-border ) 
                  :background :white
                  :accessor main-layout)
   ;;layout to make hte mouse stay in the lower part.
   (top-layout capi:pinboard-layout
               '()
               :background :white
               :input-model '(((:motion) process-mouse))
               :accessor top-layout
               :visible-min-width 1024
               :visible-min-height 400
               )

   ;;;fixation-layout: 
   (fixation-layout capi:pinboard-layout
                    '(fixation sub-circle typ-circle)
                    :accessor fixation-layout
                    :input-model '(((:gesture-spec) eyelink:press-eyelink-key))
                    :background :white
                    :visible-min-width 1024
                    :visible-min-height 768)
   (drift-layout capi:pinboard-layout
                    '(drift-target)
                    :accessor drift-layout
                    :input-model '(((:gesture-spec) eyelink:press-eyelink-key))
                    :background :white
                    :visible-min-width 1024
                    :visible-min-height 768)
   
   ;;;msg-layout
   (msg-layout capi:pinboard-layout 
               '(msg)
               :accessor msg-layout
               :input-model '(((:character) process-spacebar))
               :background :white
               :visible-min-width 1024
               :visible-min-height 768)

   ;;;subtraction-layout
   (subtraction-layout capi:switchable-layout ;subtraction-layout, can be empty, input, correct or wrong
               '(empty-pane-subtraction subtraction-input-layout subtraction-correct-layout subtraction-wrong-layout) 
               :accessor subtraction-layout
               :x 195
               :y 100)

   ;;;subtraction-input-layout
   (subtraction-input-layout capi:pinboard-layout ;input layout for subtraction
                             '(input-screen-subtraction subtraction-termA subtraction-termB subtraction-line subtraction-minus subtraction-signal-layout subtraction-input-border)
                             :accessor subtraction-input-layout
                             :background :white)

   (subtraction-signal-layout capi:switchable-layout
                              '(subtraction-empty-signal subtraction-signal)
                              :accessor subtraction-signal-layout
                              :x 40
                              :y 0)
   ;;;subtraction-correct-layout
   (subtraction-correct-layout capi:pinboard-layout ;subtraction layout for when answer is correct
                            '(subtraction-correct)
                            :accessor subtraction-correct-layout
                             :background :white)
   (subtraction-wrong-layout capi:pinboard-layout ;subtraction layout for when answer is wrong
                            '(subtraction-wrong)
                            :accessor subtraction-wrong-layout
                            :background :white)
   (keyboard-layout capi:switchable-layout
                    '(empty-pane-keyboard alphabet)
                    :accessor keyboard-layout
                    :x 625
                    :y 550)
   (numpad-layout capi:switchable-layout
                  '(empty-pane-numpad numpad)
                  :accessor numpad-layout
                  :x 130
                  :y 550
                  )
   (typing-layout capi:switchable-layout ;typing layout, input, empty, correct & wrong layouts
               '(empty-pane-typing typing-input-layout typing-correct-layout typing-wrong-layout)
               :accessor typing-layout
               :x 700
               :y 140
               )
   (typing-correct-layout capi:pinboard-layout ;layout to show when word is correct
                       '(typing-correct)
                       :accessor typing-correct-layout
                        :background :white)
   (typing-wrong-layout capi:pinboard-layout ;layot for when it's wrong
                        '(typing-wrong)
                        :accessor typing-wrong-layout
                         :background :white)
   (typing-input-layout capi:pinboard-layout ;input stuff on typing layout
                        '(input-screen-typing)
                        :accessor typing-input-layout
                         :background :white))

  (:default-initargs :title "Experiment"
   :visible-min-width 1024
   :visible-min-height 768
   :x 0
   :y 0
  ))

(defun test-interface ()
  (create-st-window)
  (switch-meta-layout 'experiment)
  (switch-keyboard-layout 'input)
  (switch-numpad-layout 'input)
  (switch-subtraction-layout 'input)
  (switch-typing-layout 'input)

)


;;set signal (or not)
(defun set-signal (val)
  (if val
      (capi:apply-in-pane-process
       (subtraction-signal *expwin*) #'(setf capi:display-pane-text) "1" (subtraction-signal *expwin*))
      (capi:apply-in-pane-process
       (subtraction-signal *expwin*) #'(setf capi:display-pane-text) "|" (subtraction-signal *expwin*)))
  (if (equalp *sub_step* 0) 
      (capi:apply-in-pane-process
       (subtraction-signal *expwin*) #'(setf capi:display-pane-text) "|" (subtraction-signal *expwin*)))
  (when *model*
    (setf (text *signal-feature*) (if val "1" "|"))
    (if (equalp *sub_step* 0) (setf (text *signal-feature*) "|"))
    (proc-display)))

;;;change the text in the pane text-input-pane
(defun set-input-text (pane text)
  (capi:apply-in-pane-process 
   pane #'(setf capi:text-input-pane-text) text pane))

;;;change text in typing input pane
(defun set-typing-text (text)
  (when *model*
    (setf (text *typing-input-feature*) text)
    (proc-display))
  (set-input-text (input-screen-typing *expwin*) (string-upcase text)))

;;;change text in subtraction input pane
(defun set-subtraction-text (text)
   (let ((t4 text))
   (capi:apply-in-pane-process 
     (input-screen-subtraction *expwin*) #'(setf capi:display-pane-text) t4 (input-screen-subtraction *expwin*))))

;;;set term A text (requires 10 digit input!!!)
(defun set-term-A (text)
  (let ((t2 text) ;convert numbers to a string first
        (t3 ""))
    (if (> (length t2) 10) (setf t2 (subseq t2 1 11)))
    (dotimes (i (length t2)) ;again push spaces in between characters
      (setf t3 (concatenate 'string t3 (subseq t2 i (+ i 1)) " ")))
    (setf t3 (subseq t3 0 (- (length t3) 1)))
    (capi:apply-in-pane-process 
     (subtraction-termA *expwin*) #'(setf capi:display-pane-text) t3 (subtraction-termA *expwin*))
    (if *model* (setf (text *subtraction-A-feature*)  t3)) 
    (if *model* (proc-display))))

;;;same as term A for term B
(defun set-term-B (text)
  (let ((t2 text)
        (t3 ""))
       (if (> (length t2) 10) (setf t2 (subseq t2 1 11)))
    (dotimes (i (length t2))
      (setf t3 (concatenate 'string t3 (subseq t2 i (+ i 1)) " ")))
    (setf t3 (subseq t3 0 (- (length t3) 1)))
    (capi:apply-in-pane-process 
     (subtraction-termB *expwin*) #'(setf capi:display-pane-text) t3 (subtraction-termB *expwin*))
    (if *model* (setf (text *subtraction-B-feature*) t3))
    (if *model* (proc-display))))

;;;set the text of subtraction wrong
(defun set-subtraction-wrong-text (text)
  (setf text (format nil "~a" text))
  (capi:apply-in-pane-process
   (subtraction-wrong *expwin*) #'(setf capi:display-pane-text) text (subtraction-wrong *expwin*)))

;;;set the text of typing wrong
(defun set-typing-wrong-text (text)
  (setf text (format nil "~a" text))
  (capi:apply-in-pane-process
   (typing-wrong *expwin*) #'(setf capi:display-pane-text) text (typing-wrong *expwin*)))

;;;get current text
(defun get-input-text (pane)
   (capi:text-input-pane-text pane))

;;;get current typing text
(defun get-typing-text ()
  (get-input-text (input-screen-typing *expwin*)))

(defun mask-next ()
  (set-term-A "#")
  (set-term-B "#")
  (set-subtraction-text ""))


;;;get current subtraction text
(defun get-subtraction-text ()
  (get-input-text (input-screen-subtraction *expwin*)))

;;;get termA 
(defun get-termA-text ()
  (get-input-text (subtraction-termA *expwin*)))

;;;get termB
(defun get-termB-text ()
  (get-input-text (subtraction-termB *expwin*)))

;;;enable/disable typing: enable all buttons and text input pane
(defun enable-typing (val)
  (setf (typing-enabled *exp*) val)
  (when (equalp
         (capi:switchable-layout-visible-child (typing-layout *expwin*)) 
         (typing-input-layout *expwin*))
    (if (and val (typing-stimulus *exp*) (equalp (typing-level *exp*) 'easy)) (set-typing-text (subseq (typing-stimulus *exp*) *typ_step* (+ *typ_step* 1))))
    (if (and val (typing-stimulus *exp*) (equalp (typing-level *exp*) 'hard) (equalp *typ_step* 0)) (set-typing-text (typing-stimulus *exp*)))
    (if val
        (capi:set-button-panel-enabled-items (alphabet *expwin*)
                                             :enable (list "A" "B" "C" "D" "E" "F" "G" "H" "I" "J" "K" 
                                                           "L" "M" "N" "O" "P" "Q" "R" "S" "T" "U" "V" "W" "X" "Y" "Z"))
      (capi:set-button-panel-enabled-items (alphabet *expwin*)
                                           :disable (list "A" "B" "C" "D" "E" "F" "G" "H" "I" "J" "K" 
                                                          "L" "M" "N" "O" "P" "Q" "R" "S" "T" "U" "V" "W" "X" "Y" "Z")))
    (capi:apply-in-pane-process 
     (input-screen-typing *expwin*) #'(setf capi:text-input-pane-enabled) val (input-screen-typing *expwin*))
    (capi:apply-in-pane-process
     (input-screen-typing *expwin*) #'(setf capi:simple-pane-foreground) (if val :black :grey) (input-screen-typing *expwin*))
    (when *model*
      (setf *typing-on* val)
      (proc-display))))


;;;enable/disable subtractoin: enable subtraction input pane
(defun enable-subtraction (val)
  (setf (subtraction-enabled *exp*) val)
  (when (equalp 
         (capi:switchable-layout-visible-child (subtraction-layout *expwin*)) 
         (subtraction-input-layout *expwin*))
    (when (subtraction-stimulus *exp*)
      (let ((A (write-to-string (first (subtraction-stimulus *exp*))))
            (B (write-to-string (second (subtraction-stimulus *exp*)))))
        (if (not (equalp *sub_step* 0)) 
            (if (>= (parse-integer (subseq A (- 10 *sub_step*) (- 11 *sub_step*))) 
                    (parse-integer (subseq B (- 10 *sub_step*) (- 11 *sub_step*)))) (set-signal nil) (set-signal t)))
        (if (equalp *sub_step* 0) (set-signal nil))
        (if val (set-term-A (subseq A (- 9 *sub_step*) (- 10 *sub_step*))))
        (if val (set-term-B (subseq B (- 9 *sub_step*) (- 10 *sub_step*))))))
    (if val
        (capi:set-button-panel-enabled-items (numpad *expwin*)
                                             :enable (list "1" "2" "3" "4" "5" "6" "7" "8" "9" "0")) 
      (capi:set-button-panel-enabled-items (numpad *expwin*)
                                           :disable (list "1" "2" "3" "4" "5" "6" "7" "8" "9" "0")))
    (capi:apply-in-pane-process
     (subtraction-termA *expwin*) #'(setf capi:simple-pane-foreground) (if val :black :grey) (subtraction-termA *expwin*))
    (capi:apply-in-pane-process
     (subtraction-termB *expwin*) #'(setf capi:simple-pane-foreground) (if val :black :grey) (subtraction-termB *expwin*))
    (capi:apply-in-pane-process
     (subtraction-signal *expwin*) #'(setf capi:simple-pane-foreground) (if val :black :grey) (subtraction-signal *expwin*))
    (capi:apply-in-pane-process
     (input-screen-subtraction *expwin*) #'(setf capi:simple-pane-foreground) (if val :black :grey) (input-screen-subtraction *expwin*))
    (when *model*
      (setf *subtraction-on* val)
      (proc-display))))

(defun enable-signal (val)
  (if val
      (capi:apply-in-pane-process 
       (subtraction-signal-layout *expwin*) #'(setf capi:switchable-layout-visible-child) 
       (subtraction-signal *expwin*) (subtraction-signal-layout *expwin*))
      (capi:apply-in-pane-process 
       (subtraction-signal-layout *expwin*) #'(setf capi:switchable-layout-visible-child) 
       (subtraction-empty-signal *expwin*) (subtraction-signal-layout *expwin*))))

;;;switch to different pane in typing layout, options:'input 'empty 'correct 'wrong
(defun switch-typing-layout (switch-to)
  (case switch-to
    ('input
     (capi:apply-in-pane-process 
      (typing-layout *expwin*) #'(setf capi:switchable-layout-visible-child) 
      (typing-input-layout *expwin*) (typing-layout *expwin*))
      (enable-typing t)) 
    ('empty
     (capi:apply-in-pane-process 
      (typing-layout *expwin*) #'(setf capi:switchable-layout-visible-child) 
      (empty-pane-typing *expwin*) (typing-layout *expwin*))
     (if *model* (setf *typing-on* nil)))
    ('correct
     (capi:apply-in-pane-process 
      (typing-layout *expwin*) #'(setf capi:switchable-layout-visible-child) 
      (typing-correct-layout *expwin*) (typing-layout *expwin*))
     (if *model* (setf *typing-on* nil)))
    ('wrong
     (capi:apply-in-pane-process 
      (typing-layout *expwin*) #'(setf capi:switchable-layout-visible-child) 
      (typing-wrong-layout *expwin*) (typing-layout *expwin*))
     (if *model* (setf *typing-on* nil))))
  (if *model* (proc-display)))

;;;switch to different pane in subtraction layout, options:'input 'empty 'correct 'wrong
(defun switch-subtraction-layout (switch-to)
  (case switch-to
    ('input
     (capi:apply-in-pane-process 
      (subtraction-layout *expwin*) #'(setf capi:switchable-layout-visible-child) 
      (subtraction-input-layout *expwin*) (subtraction-layout *expwin*))
     (enable-subtraction t))
    ('empty
     (capi:apply-in-pane-process 
      (subtraction-layout *expwin*) #'(setf capi:switchable-layout-visible-child) 
      (empty-pane-subtraction *expwin*) (subtraction-layout *expwin*))
     (if *model* (setf *subtraction-on* nil)))
    ('correct
     (capi:apply-in-pane-process 
      (subtraction-layout *expwin*) #'(setf capi:switchable-layout-visible-child) 
      (subtraction-correct-layout *expwin*) (subtraction-layout *expwin*))
     (if *model* (setf *subtraction-on* nil)))
    ('wrong
     (capi:apply-in-pane-process 
      (subtraction-layout *expwin*) #'(setf capi:switchable-layout-visible-child) 
      (subtraction-wrong-layout *expwin*) (subtraction-layout *expwin*))
     (if *model* (setf *subtraction-on* nil))))
  (if *model* (proc-display)))

(defun switch-keyboard-layout (switch-to)
  (case switch-to
    ('input
     (capi:apply-in-pane-process 
      (keyboard-layout *expwin*) #'(setf capi:switchable-layout-visible-child) 
      (alphabet *expwin*) (keyboard-layout *expwin*)))
    ('empty
     (capi:apply-in-pane-process 
      (keyboard-layout *expwin*) #'(setf capi:switchable-layout-visible-child) 
      (empty-pane-keyboard *expwin*) (keyboard-layout *expwin*)))))

(defun switch-numpad-layout (switch-to)
  (case switch-to
    ('input
     (capi:apply-in-pane-process 
      (numpad-layout *expwin*) #'(setf capi:switchable-layout-visible-child) 
      (numpad *expwin*) (numpad-layout *expwin*)))
    ('empty
     (capi:apply-in-pane-process 
      (numpad-layout *expwin*) #'(setf capi:switchable-layout-visible-child) 
      (empty-pane-numpad *expwin*) (numpad-layout *expwin*)))))


;;;switch meta-layout
(defun switch-meta-layout (switch-to)
  (case switch-to
    ('fixation
     (capi:apply-in-pane-process 
      (meta-layout *expwin*) #'(setf capi:switchable-layout-visible-child) 
      (fixation-layout *expwin*) (meta-layout *expwin*)))
    ('drift
     (capi:apply-in-pane-process 
      (meta-layout *expwin*) #'(setf capi:switchable-layout-visible-child) 
      (drift-layout *expwin*) (meta-layout *expwin*)))
    ('msg
     (capi:apply-in-pane-process 
      (meta-layout *expwin*) #'(setf capi:switchable-layout-visible-child) 
      (msg-layout *expwin*) (meta-layout *expwin*)))
    ('experiment
     (capi:apply-in-pane-process 
      (meta-layout *expwin*) #'(setf capi:switchable-layout-visible-child) 
      (main-layout *expwin*) (meta-layout *expwin*)))))




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Experiment code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;create window, display and return
(defun create-st-window () 
  (let ((interfaces (capi:collect-interfaces 'st-window))) ;destroy all old st-window interfaces
    (if interfaces (mapcar #'(lambda (interface) 
                               (capi:apply-in-pane-process interface 'capi:destroy interface))
                           interfaces)))
  
  (setf *expwin* (make-instance 'st-window)) ;create a new st-window object
  (capi:display *expwin*);display the window
  *expwin*)

;stops the mouse from going above 400 pix
(defun activate-mouse-block ()
  (capi:activate-pane (top-layout *expwin*)))



;;;;;;;;;;;;;;;;;; 
;;; Event handling
;;;;;;;;;;;;;;;;;;

(defparameter *key* nil)


;;process-mouse
(defun process-mouse (pane x y)
  (declare (ignore pane)
           (ignore y))
  (if (not *model*) (set-mouse-pos x 500)))

(defun process-spacebar (pane x y button) 
  (declare (ignore pane)
           (ignore x)
           (ignore y))
  (when (equalp button #\Space)
    (switch-meta-layout 'fixation)
    (mp:process-run-function "new-trial" nil 'start-new-trial *exp*)
    ))

(defun process-keypresses (key button)
  (declare (ignore button))
  (format t "~&Key ~a pressed (experiment)!" key)
  
  (when (and
         (subtraction-enabled *exp*) ;there is subtraction is enabled
         (subtraction-stimulus *exp*)); (enable-subtraction t))and there's a stimulus
    (format t "~&Process key-press")
    (let ((time (my-get-time)))
      (if (and (null (typing-enabled *exp*)) (not (equalp (typing-level *exp*) 'none))) (enable-typing t))
      (setf (subtraction-response *exp*) (format nil "~a~a" key (subtraction-response *exp*))) ;increase sbutraction response
      (incf *sub_step*)
      ;(set-subtraction-text (subtraction-response *exp*)) ;change the text in the window
      (print-outfile "( ~a ~a ~a 4 ~a ~a ~a key ~a ~a ~a)" ;and print to outfile
                     time
                     (- time (start-time *exp*)) 
                     (my-tracker-time)
                     (subtraction-level *exp*)
                     (typing-level *exp*)
                     (if *signal* 'on 'off)
                     key
                     (if (equalp 	
                               (subseq (subtraction-response *exp*) 0 1)
                               (subseq (subtraction-answer *exp*) (- 10 (length (subtraction-response *exp*))) (- 11 (length (subtraction-response *exp*)))))
                              1 0)
                     *trial*)
      (when (not (>= (length (subtraction-response *exp*)) 10))
        (enable-subtraction nil)
        (mask-next) ;disable subtraction
        (if (equalp (typing-level *exp*) 'none) (enable-subtraction t)))
      

      (when (>= (length (subtraction-response *exp*)) 10) ;when the length of the respone is 10 or longer (shouldn't happen)
        (if (not (equalp (typing-level *exp*) 'none)) (enable-typing t))
        (print-outfile "( ~a ~a ~a 5 ~a ~a ~a finish-subtraction ~a ~a)" ;print finished plus response
                       time
                       (- time (start-time *exp*))
                       (my-tracker-time) 
                       (subtraction-level *exp*)
                       (typing-level *exp*)
                       (if *signal* 'on 'off)
                       (subtraction-response *exp*)
                       *trial*)
        ;check if response is correct
        (cond ;if correct:
         ((equalp (- (first (subtraction-stimulus *exp*)) (second (subtraction-stimulus *exp*))) (parse-integer (subtraction-response *exp*)))
          (switch-subtraction-layout 'correct)) ;switch the layout to correct
         (t ;if not correct
          (let ((mistakes (diff (write-to-string (- (first (subtraction-stimulus *exp*)) (second (subtraction-stimulus *exp*))))
                                (subtraction-response *exp*)))) 
            (set-subtraction-wrong-text (format nil "~a ~a." mistakes (if (equalp mistakes 1) "fout" "fouten"))) ;set the text right
            (switch-subtraction-layout 'wrong))))
        
        (setf (subtraction-stimulus *exp*) nil)
        (when (and 
               (null (typing-stimulus *exp*)) ;when there are no stimuli anymore
               (null (subtraction-stimulus *exp*)))
          (mp:process-run-function "end-and-start-trial" nil 'end-trial *exp*)))))) ;and set stimulus to nil, meaning subtraction is finished

(defun process-input-button (data button)
  (declare (ignore button))
  (let ((time (my-get-time))
        (exp *exp*))
    (when (typing-enabled *exp*) ;only process stuff if typing is enabled
      (if (and (null (subtraction-enabled exp)) (not (equalp (subtraction-level *exp*) 'none))) (enable-subtraction t))
      (format t "~&Input-Button ~a clicked! (experiment)" data)
      (incf *typ_step*)
      (cond 
       ((equalp (typing-level exp) 'easy)
        (set-typing-text "#")
        (if (and (equalp (subtraction-level *exp*) 'none) (< *typ_step* 10)) (enable-typing t)))
       ((equalp (typing-level exp) 'hard)		;nav-hard
        (if (equalp *typ_step* 1) 
            (set-typing-text ""))))
      (setf (typing-response exp) (format nil "~a~a" (typing-response exp) data))
      
      (if (and (not (equalp (subtraction-level *exp*) 'none))
               (not (>= (length (typing-response *exp*)) 10)))
          (enable-typing nil))
      
      (print-outfile "( ~a ~a ~a 2 ~a ~a ~a press \"~a\" ~a ~a)"
                        time
                        (- time (start-time *exp*)) 
                        (my-tracker-time)
                        (subtraction-level *exp*)
                        (typing-level *exp*)
                        (if *signal* 'on 'off)
                        data
                        (if (equalp 
                             (subseq (typing-response *exp*) (- (length (typing-response *exp*)) 1) (length (typing-response *exp*)))
                             (subseq (typing-stimulus *exp*) (- (length (typing-response *exp*)) 1) (length (typing-response *exp*))))
                            1 0)
                        *trial*)
      (when (>= (length (typing-response *exp*)) 10) ;if 10 characters were entered
        (print-outfile "( ~a ~a ~a 6 ~a ~a ~a finish-typing \"~a\" ~a)"
                       time
                          (- time (start-time *exp*)) 
                          (my-tracker-time)
                          (subtraction-level *exp*)
                          (typing-level *exp*)
                          (if *signal* 'on 'off)
                          (typing-response *exp*)
                          *trial*)
           (set-typing-text "")
           (cond
            ((equalp (typing-stimulus *exp*) (typing-response *exp*)) ;if response was correct increase points etc
             (switch-typing-layout 'correct))
            (t ;if not correct
             (let ((mistakes (edit-distance (typing-stimulus *exp*) (string-downcase (typing-response *exp*)))))
               (set-typing-wrong-text (format nil "~a ~a." mistakes (if (equalp mistakes 1) "fout" "fouten")))) ;set the text right
             (switch-typing-layout 'wrong)))
           
           (setf (typing-stimulus *exp*) nil)
           (when (and 
                  (null (typing-stimulus *exp*)) ;when there are no stimuli anymore
                  (null (subtraction-stimulus *exp*)))
             (mp:process-run-function "end-and-start-trial" nil 'end-trial *exp*)))))) ;set typing stimulus t)))))
  
  

 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;  Experiment
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defclass experiment ()		;define class
  ((window :accessor window)	;with slots: experiment window
   (start-time :accessor start-time) ;start time of experiment
   (end-time :accessor end-time) ;end-time of a round of stimuli
   (typing-level :accessor typing-level :initform "none") ;typing level (none, easy or hard)
   (subtraction-level :accessor subtraction-level :initform "none") ;subtraction level (none easy or hard)
   (busy :accessor busy :initform nil) ;busy responding to stimuli or not
   (typing-stimulus :accessor typing-stimulus :initform nil) ;current typing stimulus
   (typing-response :accessor typing-response :initform "") ;current typing response
   (subtraction-stimulus :accessor subtraction-stimulus :initform nil) ;current subtraction stimulus
   (subtraction-response :accessor subtraction-response :initform "") ;current subtraction response
   (subtraction-answer :accessor subtraction-answer :initform "")
   (typing-enabled :accessor typing-enabled :initform t) ;typing enabled yes/no
   (subtraction-delay :accessor subtraction-delay :initform 2000) ;subtraction break time
   (subtraction-enabled :accessor subtraction-enabled :initform t))) ;subtraction enabled yes/no


;;;start a new trial, display stimuli etc
(defun start-new-trial (exp)

  (when *model*
    (model-reset))

 

  (switch-meta-layout 'fixation)
  ;first drift correction
  (when *eye*
    (safe-exp
     ("start trial")
     (eyelink:send-msg (format nil "start-drift-correction ~a" (my-get-time)))
     (switch-meta-layout 'drift)
     (eyelink:drift-correction)
     (eyelink:send-msg (format nil "end-drift-correction ~a" (my-get-time)))
     (switch-meta-layout 'fixation)
     (eyelink:start-rec 1 1 0 0)
     (let ((time (my-get-time)))
       (eyelink:send-msg (format nil "start-fixation-start-trial ~a" time))
       (print-outfile "( ~a ~a ~a 21 ~a ~a ~a start-fixation-start-trial ~a)"
                      time
                      (- time (start-time *exp*))
                      (my-tracker-time)
                      (subtraction-level *exp*)
                      (typing-level *exp*)
                      (if *signal* 'on 'off)
                      *trial*))))
  (when *model*
    (let ((time (my-get-time)))
      (print-outfile "( ~a ~a ~a 21 ~a ~a ~a start-fixation-start-trial ~a)"
                     time
                     (- time (start-time *exp*))
                     (my-tracker-time)
                     (subtraction-level *exp*)
                     (typing-level *exp*)
                     (if *signal* 'on 'off)
                     *trial*)))
  

  (if *model* (proc-display))
  (if (not *model*) (mp:wait-processing-events 6) (run-full-time 6 :real-time *realtime*))

  (when *eye*
    (safe-exp
        ("start trial 2")
      (let ((time (my-get-time)))
        (eyelink:send-msg (format nil "end-fixation-start-trial ~a" time))
        (print-outfile "( ~a ~a ~a 22 ~a ~a ~a end-fixation-start-trial ~a)"
                       time
                       (- time (start-time *exp*))
                       (my-tracker-time)
                       (subtraction-level *exp*)
                       (typing-level *exp*)
                       (if *signal* 'on 'off)
                       *trial*))))
  (when *model*
    (let ((time (my-get-time)))
      (print-outfile "( ~a ~a ~a 22 ~a ~a ~a end-fixation-start-trial ~a)"
                     time
                     (- time (start-time *exp*))
                     (my-tracker-time)
                     (subtraction-level *exp*)
                     (typing-level *exp*)
                     (if *signal* 'on 'off)
                     *trial*)))
  

  (setf (busy *exp*) t)
  (incf *trial*)
  (if *signal* (enable-signal t) (enable-signal nil))
  
  (let* ((levels (pop *current-levels*))
         (sub-level (first levels))
         (typ-level (second levels)))
    (setf (subtraction-level exp) (first levels))
    (setf (typing-level exp) (second levels))
    (setf (subtraction-response exp) "") ;set both responses to ""
    (setf (typing-response exp) "")
    
  ;pop list of the *current-levels* list, and use it to set the levels of the tasks
    (case (typing-level exp) ; depending on the level, set the appropriate stimulus
      ('none
       (setf (typing-stimulus exp) nil))
      ('easy
       (setf (typing-stimulus exp) (pop *typing-stimuli-easy*))
       (set-typing-text "#"))
      ('hard
       (setf (typing-stimulus exp) (pop *typing-stimuli-hard*))
       (set-typing-text "#")))
    
    (case (subtraction-level exp) ; depending on the level, set the appropriate stimulus
      ('none
       (setf (subtraction-stimulus exp) nil))
      ('easy
       (setf (subtraction-stimulus exp) (pop *subtraction-stimuli-easy*))
       (setf (subtraction-answer exp) (write-to-string (- (first (subtraction-stimulus exp)) (second (subtraction-stimulus exp))))))
      ('hard
       (setf (subtraction-stimulus exp) (pop *subtraction-stimuli-hard*))
       (setf (subtraction-answer exp) (write-to-string (- (first (subtraction-stimulus exp)) (second (subtraction-stimulus exp)))))))
    
    (setf *sub-circle-color* 
          (case sub-level
            ('easy :darkgreen)
            ('hard :red)
            ('none :white)))

    (setf *typ-circle-color* 
          (case typ-level
            ('easy :darkgreen)
            ('hard :red)
            ('none :white)))

    (setf (color *sub-circle-feature*)
          (case sub-level
            ('easy 'green)
            ('hard 'red)
            ('none 'white)))
    (setf (color *typ-circle-feature*)
          (case typ-level
            ('easy 'green)
            ('hard 'red)
            ('none 'white))))

  (setf *circles-on* t)
  (capi:redraw-pinboard-object (sub-circle *expwin*))
  (capi:redraw-pinboard-object (typ-circle *expwin*))

  (if *model* (proc-display))

  (schedule-dummy)
  (if (not *model*) (mp:wait-processing-events 1) (run-full-time 1 :real-time *realtime*))

  (setf *sub-circle-color* :white)
  (setf *typ-circle-color* :white)
  
  (setf *circles-on* nil)
  (capi:redraw-pinboard-object (sub-circle *expwin*))
  (capi:redraw-pinboard-object (typ-circle *expwin*))

  (if *model* (proc-display))

  (if (not *model*) (sleep .5) (run-full-time .5 :real-time *realtime*))
  (sleep .5)
  
  (switch-meta-layout 'experiment)
 
  (if (not *model*) (sleep .1) (run-full-time .1 :real-time *realtime*))
  (sleep .1)

  (setf *sub_step* 0)
  (setf *typ_step* 0)

  (activate-mouse-block)
  (set-signal nil)

  (let ((time (my-get-time)))
    
    (when *eye*
      (safe-exp
       ("start trial 3")
       (eyelink:send-msg (format nil "experiment part: ~a" *exp-part*))
       (eyelink:send-msg (format nil "start pp ~a time ~a trial ~a subtraction ~a typing ~a signal ~a" *pp* time *trial* (subtraction-level *exp*) (typing-level *exp*) (if *signal* "on" "off")))))
    
    (when (not (equalp (subtraction-level exp) 'none)) ;if there's subtraction, set the window element correctly
      (set-term-A (subseq (write-to-string (first (subtraction-stimulus exp))) 9 10))
      (set-term-B (subseq (write-to-string (second (subtraction-stimulus exp))) 9 10))
      (set-subtraction-text "")
      (switch-subtraction-layout 'input)
      (enable-subtraction t)
      (enable-typing nil)
      (switch-numpad-layout 'input)
      (print-outfile "( ~a ~a ~a 0 ~a ~a ~a new-subtraction ~a ~a)"
                     time
                     (- time (start-time *exp*))
                     (my-tracker-time) 
                     (subtraction-level *exp*)
                     (typing-level *exp*)
                     (if *signal* 'on 'off)
                     (subtraction-stimulus exp)
                     *trial*))
    
    (when (not (equalp (typing-level exp) 'none)) ;if there's typing, set the window elements for that
      (switch-typing-layout 'input)
      (switch-keyboard-layout 'input)
      (print-outfile "( ~a ~a ~a 1 ~a ~a ~a new-typing \"~a\" ~a)"
                     time
                     (- time (start-time *exp*))
                     (my-tracker-time)
                     (subtraction-level *exp*)
                     (typing-level *exp*)
                     (if *signal* 'on 'off)
                     (typing-stimulus exp)
                     *trial*))

    (if (not *model*) (sleep .1) (run-full-time .1 :real-time *realtime*))
    (sleep .1)
    (if (equalp (subtraction-level exp) 'none) (enable-typing t) (enable-typing nil)))
  (schedule-dummy)
  (setf *subtraction-on* t)
  (proc-display)
  (if *model* (run-until-condition (lambda () (not (busy *exp*)))  :real-time *realtime*)))


;;;Ends a trial
(defun end-trial (exp)
  (switch-keyboard-layout 'empty)
  (switch-numpad-layout 'empty)
  (setf (busy exp) nil) ;switch off busy
  (setf (end-time exp) (my-get-time)) ;set end-time

  (when *eye*
    (safe-exp
     ("end trial")
     (eyelink:send-msg (format nil "end pp ~a time ~a trial ~a subtraction ~a typing ~a signal ~a" *pp* (my-get-time) *trial* (subtraction-level *exp*) (typing-level *exp*) (if *signal* "on" "off")))))
  (if *model* (mp:wait-processing-events 2))
  (if (not *model*) (mp:wait-processing-events 2) (run-full-time 2 :real-time *realtime*))
  (switch-subtraction-layout 'empty)
  (switch-typing-layout 'empty)
  ;and a fixation cross
  (when *eye*
    (safe-exp
     ("end trial 2")
    (let ((time (my-get-time)))
      (eyelink:send-msg (format nil "start-fixation-end-trial ~a" time))
      (print-outfile "( ~a ~a ~a 23 ~a ~a ~a start-fixation-end-trial ~a)"
                     time
                     (- time (start-time *exp*))
                     (my-tracker-time)
                     (subtraction-level *exp*)
                     (typing-level *exp*)
                     (if *signal* 'on 'off)
                     *trial*))))
  (when *model*
    (let ((time (my-get-time)))
      
      (print-outfile "( ~a ~a ~a 23 ~a ~a ~a start-fixation-end-trial ~a)"
                     time
                     (- time (start-time *exp*))
                     (my-tracker-time)
                     (subtraction-level *exp*)
                     (typing-level *exp*)
                     (if *signal* 'on 'off)
                     *trial*)))
  (switch-meta-layout 'fixation)
  (flush-buffer)
  (gc-if-needed)
  (if (not *model*) (mp:wait-processing-events 2) (run-full-time 2 :real-time *realtime*))
  (when *eye*
    (safe-exp
     ("end trial 3")
    (let ((time (my-get-time)))
      (eyelink:send-msg (format nil "end-fixation-end-trial ~a" time))
      (print-outfile "( ~a ~a ~a 24 ~a ~a ~a end-fixation-end-trial ~a)"
                     time
                     (- time (start-time *exp*))
                     (my-tracker-time)
                     (subtraction-level *exp*)
                     (typing-level *exp*)
                     (if *signal* 'on 'off)
                     *trial*)
      (eyelink:stop-rec))))
  (when *model*
    (let ((time (my-get-time)))
      (print-outfile "( ~a ~a ~a 24 ~a ~a ~a end-fixation-end-trial ~a)"
                     time
                     (- time (start-time *exp*))
                     (my-tracker-time)
                     (subtraction-level *exp*)
                     (typing-level *exp*)
                     (if *signal* 'on 'off)
                     *trial*)))
  (if (null *current-levels*)
      (mp:process-run-function "exp-end" nil 'experiment-end *exp*)
      (mp:process-run-function "new-trial" nil 'start-new-trial *exp*)))


;;;creates an experiment
(defun create-experiment ()			
  (let ((exp (make-instance 'experiment)))	;object instance 'exp'
    (setf (window exp)      (create-st-window)	;set exp params, with a new window
          (start-time exp)  (my-get-time)       ;the current time
          (end-time exp)    (my-get-time))	;and the current time again
    (if *model* 
        (progn 
          (create-features)
          (install-device (window exp))))
    (setf *exp* exp)
    exp))                                     ;return exp-object

;;;initialize expeirment
(defmethod experiment-initialize ((exp experiment)) ;initialize everything
  (setf *exp* exp)                           ;set *exp* to exp
  (setf *expwin* (window exp))
  (when *eye*
    (set-error-exp (format nil "errors_exp_~a_~a" *exp-part* *pp*))
    (safe-exp
     ("ini")
      (eyelink:set-error (format nil "errors_~a_~a" *exp-part* *pp*))
      (eyelink:initialize-eyetracker (format nil "~a~a" *exp-part* *pp*))
      (eyelink:calibrate)
      (eyelink:send-msg (format nil "experiment pp ~a time ~a" *pp* (my-get-time) *trial*)))))             

(defmethod experiment-run ((exp experiment))	;run experiment exp
  (switch-meta-layout 'fixation) ;make sure it's on fixation before attempting calibration
  (mp:wait-processing-events 1)
  (experiment-initialize exp)		;initialize first
  (initialize-outfile)                        ;ini outfile, set path to write to
  (if (not *model*) (switch-meta-layout 'msg) (mp:process-run-function "new-trial" nil 'start-new-trial *exp*)))
  
 
;;; end experiment : destroy window
;;;
;;;

(defmethod experiment-end ((exp experiment))
  (if (not *model*) (sleep 1) (run-full-time 1 :real-time *realtime*))
  (finalize-outfile)
  (capi:destroy (window *exp*))
  
  (safe-exp
   ("close")
   (if *eye* (eyelink:close-eyetracker)))  

  (if (and (>= *block* 8) (not *model*)) (capi:display-message "Dat was het, bedankt voor je medewerking!") (if (not *model*) (experiment)))
  (if *model* (setf *modelexpdone* t)))


;;;start exp
(defun e (&optional name)
  (setf *exp-part* name)
  (let* ((exp (create-experiment))) ;create exp
    (experiment-run exp) ;run exp
  );  (experiment-end exp));close window and go back to listener  
)

;;;;;;;;;;;;;;;;;;;;;
;;; Run-Experiment
;;;;;;;;;;;;;;;;;;;;;


(defun ex ()
  (kill-toolbar)
  (experiment))


(defun experiment ()
  
  (case *block*
    (0
      (setup-pp)
      (setf *eye* nil)
      (setf *exp-part* "pSE")
      (setf *signal* (if (oddp *pp*) t nil))
      (setf *trial* 0)
     ;set correct params
      (setf *current-levels*
            '(
              (easy none)
              (easy none)
              ))
     
     (capi:display-message "Je gaat nu de makkelijke aftreksommen oefenen. Klik op OK om te beginnen.")
     (e "pSE"))
    (1
      (setf *exp-part* "pSH")
      (setf *eye* nil)
      (setf *signal* (if (oddp *pp*) t nil))
      ;set correct params
      (setf *current-levels*
            '(
              (hard none)
              (hard none)
              ))
     
     (capi:display-message "Je gaat nu de moeilijke aftreksommen oefenen. Klik op OK om te beginnen.")
     (e "pSH"))
    (2
      (setf *exp-part* "pTE")
      (setf *eye* nil)
      (setf *signal* (if (oddp *pp*) t nil))
      ;set correct params
      (setf *current-levels*
            '(
              (none easy)
              (none easy)
              ))
     
     (capi:display-message "Je gaat nu de makkelijke type-taak oefenen. Klik op OK om te beginnen.")
     (e "pTE"))
    (3
      (setf *exp-part* "pTH")
      (setf *eye* nil)
      (setf *signal* (if (oddp *pp*) t nil))
      ;set correct params
      (setf *current-levels*
            '(
              (none hard)
              (none hard)
              ))
     
     (capi:display-message "Je gaat nu de moeilijke type-taak oefenen. Klik op OK om te beginnen.")
     (e "pTH"))
    (4
     (setf *exp-part* "pboth")
     (setf *eye* t)
     (setf *signal* (if (oddp *pp*) t nil))
   
     ;set correct params
     (setf *current-levels*
           '(
             (easy easy)
             (hard easy)
             (easy hard)
             (hard hard)
             ))
     
     (capi:display-message "Je gaat nu beide taken tegelijkertijd oefenen. Klik op OK om te beginnen.")
     (e "pboth"))
    (5
     (setf *exp-part* "real1")
     (setf *eye* t)
     (setf *trial* 0)
     (setf *signal* (if (oddp *pp*) t nil))
     ;set correct params
     (setf *current-levels*
           (get-block))
     
     (capi:display-message "Het echte experiment begint nu, halverwege krijg je pauze. Veel succes! Klik op OK om te beginnen.")
     (e "real1"))
    (6
      (setf *exp-part* "pSH2")
      (setf *signal* (if (oddp *pp*) nil t))
      (setf *eye* nil)
      ;set correct params
      (setf *current-levels*
            '(
              (hard none)
              (hard none)
              ))
     (capi:display-message "Je bent halverwege, je kunt nu pauze nemen. Laat dit aan de onderzoeker weten. Als je er klaar voor bent, klik dan op OK om te beginnen.")
     (capi:display-message "Je gaat nu de moeilijke subtraction-taak nog een keer oefenen. Klik op OK om te beginnen.")
     (e "pSH2"))
    (7
     (setf *exp-part* "real2")
     (setf *eye* t)
     (setf *trial* 28)
     (setf *signal* (if (oddp *pp*) nil t))
     ;set correct params
     (setf *current-levels*
           (get-block))
     (capi:display-message "Nu ga je het laatste blok uitvoeren, klik op OK om te beginnen.")
     (e "real2"))
    )

  (incf *block*))

(defun get-block ()
  (randomize
   '(
     (easy easy)
     (easy hard)
     (hard easy)
     (hard hard)
     (easy easy)
     (easy hard)
     (hard easy)
     (hard hard)
     (easy easy)
     (easy hard)
     (hard easy)
     (hard hard)
     (easy easy)
     (easy hard)
     (hard easy)
     (hard hard)
     (easy easy)
     (easy hard)
     (hard easy)
     (hard hard)
     (easy easy)
     (easy hard)
     (hard easy)
     (hard hard)
     (easy easy)
     (easy hard)
     (hard easy)
     (hard hard)
)))






;########################################################################
;### ACT-R interfacing
;########################################################################

;;;
;;; To start we will define the general motor methods needed for the device.
;;; In this case we will be moving the mouse so set the position methods
;;; appropriately.



(defmethod device-move-cursor-to ((device st-window) loc)
  (setf (mouse-pos device) loc))

(defmethod get-mouse-coordinates ((device st-window))
  (mouse-pos device))

(defmethod device-handle-click ((device st-window))
  (let* ((x (aref (get-mouse-coordinates *expwin*) 0))
         (y (aref (get-mouse-coordinates *expwin*) 1))
         (button nil))
    (if *subtraction-on*
        (progn
          (dolist (i *numpad-buttons*)
            (if (and (equalp x (x i)) (equalp y (y i)))
                (setf button i)))
          (process-keypresses (label button) button)
          (format t "subtraction toch"))
      (progn ;*typing-on*
        (dolist (i *alphabet-buttons*)
          (if (and (equalp x (x i)) (equalp y (y i)))
              (setf button i)))
        (process-input-button (label button) button)))))


;(defmethod device-handle-keypress ((device st-window) key)
;  (process-keypresses "actr" 0 0 key))

(defmethod device-speak-string ((devices st-window) string)
  ;; ignore model's speech output
  nil)

;;; Now we deal with the visual processing

;;; Still ignoring the mouse cursor for visual processing.

(defmethod cursor-to-vis-loc ((device st-window))
  nil)

;;; This method gets called during proc-display to generate
;;; the features available to the model.  It must return a 
;;; list (or list of lists) of visual-location (or sub-type of 
;;; visual-location) chunk names.  

;;; Here, we're calling specific build-vis-locs-for that
;;; are defined on the specific subviews and then just
;;; collecting those into a list to return.

(defmethod build-vis-locs-for ((device st-window) vis-mod)
  (setf (subs device) nil)
  (when *typing-on* 
    (dolist (x *alphabet-buttons*) (push x (subs device)))
    (if (> (length (text *typing-input-feature*)) 0) (push *typing-input-feature* (subs device))))
  (when *subtraction-on*
    (push *subtraction-A-feature* (subs device))
    (push *subtraction-B-feature* (subs device))
    (dolist (x *numpad-buttons*) (push x (subs device))))
  (when (not (equalp *sub-circle-color* :white))
    (push *sub-circle-feature* (subs device))
    (push *typ-circle-feature* (subs device)))
  (when (and *signal* *subtraction-on*)
    (push *signal-feature* (subs device)))

  (let ((feats nil))
    (dolist (x (subs device) feats)
      (push (build-vis-locs-for x vis-mod) feats))))


(defmethod build-vis-locs-for ((view text-feature) vis-mod)
  (let ((vl (car (define-chunks-fct `((isa visual-location
                                           color ,(color view)
                                           screen-x ,(x view)
                                           screen-y ,(y view)
                                           height ,(height view)
                                           width ,(width view)
                                           kind ,(kind view)
                                           value ,(kind view)))))))
    
    
    (setf (chunk-real-visual-value vl) (text view))
    (setf (chunk-visual-object vl) view)
    vl))

(defmethod build-vis-locs-for ((view button-feature) vis-mod)
  (let ((vl (car (define-chunks-fct `((isa visual-location
                                           color ,(color view)
                                           screen-x ,(x view)
                                           screen-y ,(y view)
                                           height ,(height view)
                                           width ,(width view)
                                           kind ,(kind view)
                                           value ,(kind view)
                                           ))))))
        
    (setf (chunk-real-visual-value vl) (label view))
    (setf (chunk-visual-object vl) view)
    vl))

(defmethod build-vis-locs-for ((view circle-feature) vis-mod)
  (let ((vl (car (define-chunks-fct `((isa visual-location
                                           color ,(color view)
                                           screen-x ,(x view)
                                           screen-y ,(y view)
                                           height ,(height view)
                                           width ,(width view)
                                           kind ,(kind view)
                                           value ,(kind view)
                                           ))))))
        
    (setf (chunk-real-visual-value vl) 'circle)
    (setf (chunk-visual-object vl) view)
    vl))


;(defmethod vis-loc-to-obj ((view text-feature) vis-loc)
;  (let ((new-object (car (define-chunks-fct `((isa ,(chunk-slot-value-fct vis-loc 'kind)))))))
;    (fill-default-vis-obj-slots new-object vis-loc)
;        (set-chunk-slot-value-fct new-object 'x (x view))
;        (set-chunk-slot-value-fct new-object 'y (y view))
;        new-object))




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;for seeing the focus ring
(defmethod update-me ((object capi:pinboard-object) (device capi:interface) xyloc)
  (if  (process-parameters (act-r-parameter-owner (get-parameter-struct :show-focus)) :show-focus)
      (let ((pb  (capi:switchable-layout-visible-child (meta-layout *expwin*))))
        (capi:execute-with-interface device
                                     #'(lambda (pb object  x y )
                                  ;   (capi:manipulate-pinboard
                                   ;   pb object :add-top )
                                         (setf (capi:pinboard-pane-position object)
                                           (values x y))
                                     (unless (capi:pinboard-object-pinboard object)
                                       (capi:manipulate-pinboard
                                        pb object :add-top )))
                                 pb object
                                 (- (svref xyloc 0) (object-x-adjustment object))
                                 (- (svref xyloc 1) (object-y-adjustment object))
                                 ))))
